# coding=utf-8
import csv
import json

import os
from autosubmit.config.log import Log

from earthdiagnostics.constants import Basins
from earthdiagnostics.domain import Domain


class VariableJsonException(Exception):
    pass


class SingletonType(type):
    def __call__(cls, *args):
        try:
            return cls.__instance
        except AttributeError:
            cls.__instance = super(SingletonType, cls).__call__(*args)
            return cls.__instance


class VariableList(object):
    __metaclass__ = SingletonType

    def __init__(self):
        self._dict_variables = {}

    def get_variable(self, original_name, silent=False):
        """
        Returns the cmor variable instance given a variable name

        :param original_name: original variable's name
        :type original_name: str
        :param silent: if True, omits log warning when variable is not found
        :type silent: bool
        :return: CMOR variable
        :rtype: Variable
        """
        try:
            return self._dict_variables[original_name.lower()]
        except KeyError:
            if not silent:
                Log.warning('Variable {0} is not defined in the CMOR table. Please add it'.format(original_name))
            return None

    def load_variables(self, table_name):
        """
        Loads the  CMOR csv and creates the variables dictionary
        """
        Variable._dict_variables = dict()
        # cls._load_variable_list('default')
        self._load_variable_list(table_name)

    def _load_variable_list(self, table_name):
        csv_path = self._get_csv_path(table_name)
        if os.path.isfile(csv_path):
            self._load_file(csv_path)
            return
        json_folder = self._get_json_folder(table_name)
        if os.path.isdir(json_folder):
            self._load_json(json_folder)

    def _get_csv_path(self, table_name):
        csv_table_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                      'cmor_tables/{0}.csv'.format(table_name))
        return csv_table_path

    def _get_json_folder(self, table_name):
        json_folder = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                   'cmor_tables/{0}/Tables'.format(table_name))
        return json_folder

    def _load_file(self, csv_table_path):
        with open(csv_table_path, 'rb') as csvfile:
            reader = csv.reader(csvfile, dialect='excel')
            for line in reader:
                if line[0] == 'Variable':
                    continue

                var = Variable()
                var.parse_csv(line)
                if not var.short_name:
                    continue

                for alternative_name in line[0].split(':'):
                    alternative_name = alternative_name.lower()
                    if alternative_name in self._dict_variables:
                        old_var = self._dict_variables[alternative_name]
                        self._dict_variables[old_var.short_name.lower()] = var
                    self._dict_variables[alternative_name] = var
                self._dict_variables[var.short_name.lower()] = var

    def _load_json(self, json_folder):
        for file_name in os.listdir(json_folder):
            if file_name in ('CMIP6_grids.json', 'CMIP6_formula_terms.json'):
                continue
            json_data = open(os.path.join(json_folder, file_name)).read()
            data = json.loads(json_data)
            if 'variable_entry' in data:
                Log.debug('Parsing file {0}'.format(file_name))
                self._load_json_variables(data['variable_entry'])

    def _load_json_variables(self, json_data):
        for short_name in json_data.keys():
            variable = Variable()
            try:
                variable.parse_json(json_data[short_name], short_name)
                self._dict_variables[variable.short_name.lower()] = variable
            except VariableJsonException:
                Log.error('Could not read variable {0}'.format(short_name))


class Variable(object):
    """
    Class to characterize a CMOR variable. It also contains the static method to make the match between thje original
    name and the standard name. Requires data _convetion to be available in cmor_tables to work.
    """
    _dict_variables = None

    def __init__(self):
        self.short_name = None
        self.standard_name = None
        self.long_name = None
        self.domain = None
        self.basin = None
        self.units = None
        self.valid_min = None
        self.valid_max = None
        self.grid = None

    def parse_json(self, json_var, key):
        if 'out_name' in json_var:
            self.short_name = json_var['out_name']
        else:
            raise VariableJsonException('Variable has no out name defined'.format(key))
        self.standard_name = json_var['standard_name']
        self.long_name = json_var['long_name']

        domain = json_var['modeling_realm'].split(' ')
        if len(domain) > 1:
            Log.warning('Multiple modeling realms assigned to variable {0}: {1}. '
                        'We wil use first ({1[0]}) as domain'.format(self.short_name, domain))
        if not domain[0]:
            Log.warning('Variable {0} has no modeling realm defined'.format(self.short_name))
        else:
            self.domain = Domain(domain[0])

        self.valid_min = json_var['valid_min']
        self.valid_max = json_var['valid_max']
        self.units = json_var['units']

    def parse_csv(self, var_line):
        self.short_name = var_line[1].strip()
        self.standard_name = var_line[2].strip()
        self.long_name = var_line[3].strip()
        self.domain = Domain(var_line[4].strip())
        self.basin = Basins.parse(var_line[5])
        self.units = var_line[6].strip()
        self.valid_min = var_line[7].strip()
        self.valid_max = var_line[8].strip()
        self.grid = var_line[9].strip()


class VarType(object):
    MEAN = 1
    STATISTIC = 2

    @staticmethod
    def to_str(vartype):
        if vartype == VarType.MEAN:
            return 'mean'
        elif vartype == VarType.STATISTIC:
            return 'statistics'
        else:
            raise ValueError('Variable type {0} not supported'.format(vartype))


